/*=============================================================================
author        : Walter Schreppers
filename      : l33t.cpp
created       : 18/3/2005 at 02:11:55
modified      : 
version       : 
copyright     : Walter Schreppers
bugreport(log): 
license       : LGPL
=============================================================================*/

#include <iostream>
#include <string>
#include <vector>
#include <sstream>
#include <algorithm>

using namespace std;


#define foreach( i, c )\
  typedef __typeof__( c ) c##_CONTAINERTYPE;\
    for( c##_CONTAINERTYPE::iterator i = c.begin(); i != c.end(); ++i )


string toLower( const string& in ){
  string out = in;
  foreach( i, out ){
    if( ( *i >= 'A') && ( *i <= 'Z' ) ) *i=*i+32;
  }
  
  return out;
}


unsigned int max( unsigned int a, unsigned int b){
  if( a>b) return a;
  else return b;
}
void Replace( string& s, const string& from, const string& to, bool bReverse ){
  string From, To;
  if( bReverse ){
    From  = to;
    To    = from;
  }
  else{
    From  = from;
    To    = to;
  }
  string::size_type pos;
  while( (pos=s.find(From)) != string::npos ) 
    s.replace( pos, From.length(), To );
}


string eliteLevel1( const string& input, bool bReverse ){
  string out;
  if(!bReverse ) out = toLower( input );
  else out = input;
  
  Replace(out, "a", "4",  bReverse );
  Replace(out, "e", "3",  bReverse );
  Replace(out, "i", "I",  bReverse );
  Replace(out, "o", "0",  bReverse );
  Replace(out, "f", "pH", bReverse );
  
  // if( out[ out.length()-1 ] == 's' ) out[ out.length()-1 ] = 'z'; //plurals to 'z' but does not work, because does becomes doez
  
  return out;
}

string eliteLevel2( const string& input, bool bReverse ){
  string out = eliteLevel1( input, bReverse );
  Replace( out, "o", "()"       , bReverse );
  Replace( out, "u", "|_|"      , bReverse );
  Replace( out, "y", "Y"        , bReverse ); //7 allready used for t ...
  Replace( out, "d", "|)"       , bReverse );
  Replace( out, "w", "\\/\\/"   , bReverse );
  Replace( out, "m", "/\\/\\"   , bReverse );
  Replace( out, "s", "5"        , bReverse );
  Replace( out, "t", "7"        , bReverse );
  Replace( out, "g", "9"        , bReverse );
  Replace( out, "l", "1"        , bReverse );

  return out;
}

void clipWord( string& s, vector<string>& words, string::size_type& pos ){
  words.push_back( string( s.substr( 0, pos ) ) );
  words.push_back( string( s.substr( pos, 1 ) ) );
  s = s.substr( pos+1, s.length()-pos );
}

void Split( string s, vector<string>& words ){
  string::size_type pos;
  while(1){
    if( (pos=s.find(" ")) != string::npos ){
      clipWord( s, words, pos );
    }
    else if( (pos=s.find("\t")) != string::npos ){
      clipWord( s, words, pos );
    }
    else if( (pos=s.find("\t")) != string::npos ){
      clipWord( s, words, pos );
    }
    else if( (pos=s.find("\n")) != string::npos ){
      clipWord( s, words, pos );
    }
    else if( (pos=s.find("\r")) != string::npos ){
      clipWord( s, words, pos );
    }
    else{
      words.push_back( s );
      return;
    }
  } 
}
string convert( const string& input, int level=1, bool bReverse=0 ){
  vector<string> words;
  string output;

  Split( input, words );
  foreach( i, words ){
    switch( level ){
      case 2  : output+=eliteLevel2( *i, bReverse ); break;
      default : output+=eliteLevel1( *i, bReverse ); break;
    } 
  }
  
  return output;
}


void showUsage( string progName ){
  cerr << "USAGE: " << progName << "[plain text] [-v] [-l <level>]" << endl;
  cerr << "  level : 1=l33t, 2=super l33t (default 1)" << endl;
  cerr << "  -v    : reverse the conversion!" <<endl;    
  cerr << " string = optional the string to convert use quotes for spaced words" << endl;
  cerr << " if no plain text string is given " << progName << " expects input on stdin." << endl;
  cerr << endl << "Written by Walter Schreppers"<<endl;
  exit(1);
}

int main(int argc, char** argv){

  if( argc<=5 ){
    string arg,input;
    int i=1;
    int level=1;
    bool bFromCin = true;
    bool bReverse = false;
    while( i< argc ){
      arg="";
      arg+=argv[i++];
      if(( arg=="-l" )||(arg=="-level")){
        string sLev = string( argv[i++] );
        istringstream is( sLev );
        is >> level;
      }
      else if( arg=="-v" ){
        bReverse = true;
      }
      else if( arg=="-help" ){
        showUsage( argv[0] );
      }
      else{
        bFromCin = false;
        input += arg;
      }
    }

    if( bFromCin ){
      int c;
      while( 1 ){
        c=cin.get();
        if( cin.eof() ) break;
        input+=c;
      }
      cout << convert( input, level, bReverse );
    }
    else{
      cout << convert( input, level, bReverse )<<endl;    
    }
  }
  else{
    showUsage( argv[0] );
  }

  return 0;
}


